/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper
 * All rights reserved.
 * Modified by Tim Edwards, April, 1997 for CIO-DIO48 board
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/***************************************************************************
 *
 *  diotest.c
 *
 *  This program is used to test the CIO-DIO48
 *  Linux loadable module (dio48).
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <dio96.h>
#include <structdef.h>
#include <constants.h>

/***************************************************************************
 *
 *  Global Data
 *
 ***************************************************************************/

#define LONG unsigned long
#define BYTE unsigned char


int open_dio_board(dio_variable_struct *dio_variables_ptr)
{	
	char *DevName[] = {"/dev/dio/dio0",
   		          "/dev/dio/dio1",
			  "/dev/dio/dio2",
			  "/dev/dio/dio3"};
	int i;
		
	//printf("System startup: Opening DIO-96 IO Board File Descriptors\n\n");
	for (i = 0; i <= 3; i++){
		if ((dio_variables_ptr->fd_chip_82C55[i] = open(DevName[i], O_RDWR)) < 0) {
			printf("error opening DIO96 device AP\n");
			exit(2);
		}
        }	
 	return(1);
}

int chip_init(dio_variable_struct *dio_variables_ptr, short *chip_init_array)
{

	int chip_mode, i;

	//CHIP 0 CONFIGURATION
	dio_variables_ptr->mode_chip_82C55_port_A[0] = chip_init_array[0];
	dio_variables_ptr->mode_chip_82C55_port_B[0] = chip_init_array[1];
	dio_variables_ptr->mode_chip_82C55_port_C_lo[0] = chip_init_array[2];
	dio_variables_ptr->mode_chip_82C55_port_C_hi[0] = chip_init_array[3];

	//CHIP 1 CONFIGURATION
	dio_variables_ptr->mode_chip_82C55_port_A[1] = chip_init_array[4];
	dio_variables_ptr->mode_chip_82C55_port_B[1] = chip_init_array[5];
	dio_variables_ptr->mode_chip_82C55_port_C_lo[1] = chip_init_array[6];
	dio_variables_ptr->mode_chip_82C55_port_C_hi[1] = chip_init_array[7];
	//CHIP 2 CONFIGURATION
	dio_variables_ptr->mode_chip_82C55_port_A[2] = chip_init_array[8];
	dio_variables_ptr->mode_chip_82C55_port_B[2] = chip_init_array[9];
	dio_variables_ptr->mode_chip_82C55_port_C_lo[2] = chip_init_array[10];
	dio_variables_ptr->mode_chip_82C55_port_C_hi[2] = chip_init_array[11];
	//CHIP 3 CONFIGURATION
	dio_variables_ptr->mode_chip_82C55_port_A[3] = chip_init_array[12];
	dio_variables_ptr->mode_chip_82C55_port_B[3] = chip_init_array[13];
	dio_variables_ptr->mode_chip_82C55_port_C_lo[3] = chip_init_array[14];
	dio_variables_ptr->mode_chip_82C55_port_C_hi[3] = chip_init_array[15];
		
	for (i = 0; i <= 3; i++){
		chip_mode = OUTPUT | (dio_variables_ptr->mode_chip_82C55_port_A[i] * CNTL_A) | (dio_variables_ptr->mode_chip_82C55_port_B[i] * CNTL_B) | (dio_variables_ptr->mode_chip_82C55_port_C_hi[i] * CNTL_CH) | (dio_variables_ptr->mode_chip_82C55_port_C_lo[i] * CNTL_CL);
		printf("Setting mode of 82C55-%c to desired input / output configuration: %X.\n", (i + 0x41), chip_mode);
		ioctl(dio_variables_ptr->fd_chip_82C55[i], (AP_SET_MODE + (0x100 * i)), chip_mode);
        }
	return(1);
}

int initialize_bits(dio_variable_struct *dio_variables_ptr, short *bit_init_array)
{
       	int i;
       	
       	//initialize all DIOs to OFF (set_hi) or ON (set_lo)
	for (i = 0; i <= 95; i++){
		if (bit_init_array[i] == 0)
			ioctl(dio_variables_ptr->fd_chip_82C55[0], dio_variables_ptr->io_set_hi[i]);
        	if (bit_init_array[i] == 1)
			ioctl(dio_variables_ptr->fd_chip_82C55[0], dio_variables_ptr->io_set_lo[i]);
	}
	//sleep(1);
	return(1);
}

int populate_ioctls(dio_variable_struct *dio_variables_ptr)
{
	int i, base;
	
	//base = AP_SET_MODE;
	
	//initialize IO hi array
	base = 0x0e0;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_hi[i] = (base + i);
	base = 0x1e0;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_hi[i + 24] = (base + i);
	base = 0x2e0;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_hi[i + 48] = (base + i);
	base = 0x3e0;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_hi[i + 72] = (base + i);

	//initialize IO low array
	base = 0x080;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_lo[i] = (base + i);
	base = 0x180;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_lo[i + 24] = (base + i);
	base = 0x280;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_lo[i + 48] = (base + i);
	base = 0x380;
	for (i = 0; i <= 23; i++)
		dio_variables_ptr->io_set_lo[i + 72] = (base + i);

	return(1);
}

int read_digital_inputs(dio_variable_struct *dio_variables_ptr, char *dio_state)
{
	int i, j;
	long temp_val;

	for (i = 0; i <= 95; i++)
		*(dio_state + i) = 0;

	temp_val = 0;
	read(dio_variables_ptr->fd_chip_82C55[0], &temp_val, 3);		
	for (i = 0, j = 0; i <= 23; i++, j++)
		*(dio_state + i) = (temp_val & (1 << j)) >> j;

	temp_val = 0;
	read(dio_variables_ptr->fd_chip_82C55[1], &temp_val, 3);		
	for (i = 24, j = 0; i <= 47; i++, j++)
		*(dio_state + i) = (temp_val & (1 << j)) >> j;

	temp_val = 0;
	read(dio_variables_ptr->fd_chip_82C55[2], &temp_val, 3);		
	for (i = 48, j = 0; i <= 71; i++, j++)
		*(dio_state + i) = (temp_val & (1 << j)) >> j;

	temp_val = 0;
	read(dio_variables_ptr->fd_chip_82C55[3], &temp_val, 3);		
	for (i = 72, j = 0; i <= 95; i++, j++)
		*(dio_state + i) = (temp_val & (1 << j)) >> j;

	return(1);
}

int write_digital_outputs(dio_variable_struct *dio_variables_ptr, int value, int bit_number)
{
	//bit_number--;
	if (value == 'H')
		ioctl(dio_variables_ptr->fd_chip_82C55[0], dio_variables_ptr->io_set_hi[bit_number]);	
	if (value == 'L')
		ioctl(dio_variables_ptr->fd_chip_82C55[0], dio_variables_ptr->io_set_lo[bit_number]);	
	return(1);
}

int close_dio_board(dio_variable_struct *dio_variables_ptr)
{	
	int i;
		
	//printf("System shutdown: Closing DIO-96 IO Board File Descriptors\n\n");
	for (i = 0; i <= 3; i++)
		close(dio_variables_ptr->fd_chip_82C55[i]);
 	return(1);
}
